/*---------------------------------------------------------------------------*\
 =========                   |
 \\      /   F ield          | OpenFOAM: The Open Source CFD Toolbox
  \\    /    O peration      |
   \\  /     A nd            | Copyright (C) 1991-2005 OpenCFD Ltd.
    \\/      M anipulation   |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

Application
    calcMassFlow

Description
    calculates Massflow through selected face sets and boundary patches

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "faceSet.H"

#include "FaceSetUtilities.H"
using namespace Foam::FaceSetUtilities;


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //
// Main program:

int main(int argc, char *argv[])
{
#   include "addTimeOptions.H"

#   include "setRootCase.H"
#   include "createTime.H"
#   include "createMesh.H"

    // Read control dictionary
    IOdictionary calcMassFlowDict
    (
        IOobject
        (
            "calcMassFlowDict",
            runTime.system(),
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    );

    const wordList bcNames = calcMassFlowDict.lookup("bcs");

    dimensionedScalar rho=dimensionedScalar("noRho",dimensionSet(0,0,0,0,0,0,0),1);

    if(calcMassFlowDict.found("rho")) {

      dimensionedScalar tmp=calcMassFlowDict.lookup("rho");
      rho.value()=tmp.value();
      rho.dimensions().reset(tmp.dimensions());

      Info << "Reseting rho to " << rho << " (Assuming incompressible case)\n" << endl;
    }

    scalar scaleFactor=1;
    if(calcMassFlowDict.found("scaleFactor")) {
        scaleFactor=readScalar(calcMassFlowDict.lookup("scaleFactor"));
    }

    // Get times list
    instantList Times = runTime.times();

    // set startTime and endTime depending on -time and -latestTime options
#   include "checkTimeOptions.H"

    for (label i=startTime; i<endTime; i++)
    {
        runTime.setTime(Times[i], i);

        Info<< "Time = " << runTime.timeName() << endl;

#       include "createPhi.H"
	
	forAll(bcNames,bcI){
	  scalar flux=0;
	  bool fluxOK=true;

	  label patchIndex=mesh.boundaryMesh().findPatchID(bcNames[bcI]);
	  if (patchIndex >= 0) {
            flux = sum(phi.boundaryField()[patchIndex]);
	  } else {
	    faceSet bcSet(mesh,bcNames[bcI],IOobject::MUST_READ,IOobject::NO_WRITE);
            autoPtr<flipSet> flips=checkFaceSetSides(mesh,bcSet);

	    if(bcSet.size()>0) {
	      int cnt=0;
	      for(faceSet::iterator faceI=bcSet.begin();faceI!=bcSet.end();faceI++) {
		cnt++;
                scalar factor=(flips()[faceI.key()] ? +1. : -1.);
		flux+=factor*phi[faceI.key()];
	      }
	    } else {
	      fluxOK=false;
	    }
	  }

	  dimensionedScalar fluxOut=scaleFactor*dimensionedScalar("",phi.dimensions(),flux)*rho;
	  fluxOut.name()="";

	  if(fluxOK) {
	    Info << "Flux at "<< bcNames[bcI] << " = " << fluxOut << endl;
	  }
	}
	Info << endl;
    }

    Info << "End\n" << endl;

    return 0;
}


// ************************************************************************* //
